package it.enea.xlab.GECORegistration.config;

import org.springframework.boot.SpringBootConfiguration;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.HibernateJpaVendorAdapter;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.annotation.EnableTransactionManagement;


import com.zaxxer.hikari.HikariDataSource;

@SpringBootConfiguration
@EnableTransactionManagement
@EnableJpaRepositories(
  basePackages = {"it.enea.xlab.GECORegistration.model.data",  
		  "it.enea.xlab.GECORegistration.service.data",
		  "it.enea.xlab.GECORegistration.service.parametricQuery",
		  "it.enea.xlab.GECORegistration.model.parametricQuery"},
  entityManagerFactoryRef = "dataEntityManagerIDP",
  transactionManagerRef = "dataTransactionManagerIDP")
@ComponentScan(basePackages = {"it.enea.xlab.GECORegistration.model.data",  
		"it.enea.xlab.GECORegistration.service.data",
		"it.enea.xlab.GECORegistration.service.parametricQuery",
		"it.enea.xlab.GECORegistration.model.parametricQuery"})
public class dataDBConfiguration{

    @Bean
    @ConfigurationProperties(prefix="spring.data-datasource-idp")
    public HikariDataSource dataDataSource() {
    		return DataSourceBuilder.create().type(HikariDataSource.class).build();
    }
    
    @Bean
    public LocalContainerEntityManagerFactoryBean dataEntityManagerIDP() {
        LocalContainerEntityManagerFactoryBean em = new LocalContainerEntityManagerFactoryBean();
        em.setPackagesToScan(new String[]{
        		"it.enea.xlab.GECORegistration.model.data",  
        		"it.enea.xlab.GECORegistration.service.data",
        		"it.enea.xlab.GECORegistration.service.parametricQuery",
        		"it.enea.xlab.GECORegistration.model.parametricQuery"});
        em.setJpaVendorAdapter(new HibernateJpaVendorAdapter());        
        em.setDataSource(dataDataSource());
        return em;
    }

    @Bean
    public PlatformTransactionManager dataTransactionManagerIDP() {
		return new DataSourceTransactionManager(dataDataSource());
    }

    @Bean(name = "jdbcTemplateData")
    public JdbcTemplate jdbcTemplateData() {
     return new JdbcTemplate(dataDataSource());
    }
    
}