package it.enea.xlab.GECORegistration.config;

import java.util.HashMap;

import javax.sql.DataSource;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringBootConfiguration;
import org.springframework.boot.autoconfigure.EnableAutoConfiguration;
import org.springframework.boot.autoconfigure.domain.EntityScan;
import org.springframework.boot.autoconfigure.jdbc.DataSourceAutoConfiguration;
import org.springframework.boot.autoconfigure.orm.jpa.HibernateJpaAutoConfiguration;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.env.Environment;
import org.springframework.data.jpa.repository.config.EnableJpaRepositories;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.orm.jpa.LocalContainerEntityManagerFactoryBean;
import org.springframework.orm.jpa.vendor.HibernateJpaVendorAdapter;
import org.springframework.transaction.PlatformTransactionManager;

@SpringBootConfiguration
@EnableJpaRepositories(
  basePackages = {
		  "it.enea.xlab.GECORegistration.model.structure", 
		  "it.enea.xlab.GECORegistration.service.structure", 
		  "it.enea.xlab.GECORegistration.model.user",  
		  "it.enea.xlab.GECORegistration.service.user"},
  entityManagerFactoryRef = "userEntityManagerIDP",
  transactionManagerRef = "userTransactionManagerIDP")

@EntityScan(basePackages = {
		  "it.enea.xlab.GECORegistration.model.structure", 
		  "it.enea.xlab.GECORegistration.model.user"})
@EnableAutoConfiguration(exclude = {DataSourceAutoConfiguration.class, HibernateJpaAutoConfiguration.class})
public class userDBConfiguration {
    
	@Autowired
    private Environment env;
	
    @Primary
    @Bean
    @ConfigurationProperties(prefix="spring.user-datasource-idp")
    public DataSource userDataSource() {
        return DataSourceBuilder.create().build();
    }
    @Bean
    @Primary
    public LocalContainerEntityManagerFactoryBean userEntityManagerIDP() {
        LocalContainerEntityManagerFactoryBean em
          = new LocalContainerEntityManagerFactoryBean();
        em.setDataSource(userDataSource());
        em.setPackagesToScan(
          new String[] { 
        		  "it.enea.xlab.GECORegistration.model.structure", 
        		  "it.enea.xlab.GECORegistration.service.structure", 
        		  "it.enea.xlab.GECORegistration.model.user",  
        		  "it.enea.xlab.GECORegistration.service.user"
        		  });

        HibernateJpaVendorAdapter vendorAdapter
          = new HibernateJpaVendorAdapter();
        em.setJpaVendorAdapter(vendorAdapter);
        HashMap<String, Object> properties = new HashMap<>();
        
		properties.put("hibernate.hbm2ddl.auto",
          env.getProperty("hibernate.hbm2ddl.auto"));
        properties.put("hibernate.dialect",
          env.getProperty("hibernate.dialect"));
        em.setJpaPropertyMap(properties);

        return em;
    }
    
    @Bean
    @Primary
    public PlatformTransactionManager userTransactionManagerIDP() {
 
        JpaTransactionManager transactionManager
          = new JpaTransactionManager();
        transactionManager.setEntityManagerFactory(
        		userEntityManagerIDP().getObject());
        return transactionManager;
    }
}