package it.enea.xlab.GECORegistration.service.identityManager;

import java.util.Arrays;
import java.util.Collections;


import javax.ws.rs.core.Response;

import org.keycloak.TokenVerifier;
import org.keycloak.admin.client.CreatedResponseUtil;
import org.keycloak.admin.client.Keycloak;
import org.keycloak.admin.client.resource.RealmResource;
import org.keycloak.admin.client.resource.UserResource;
import org.keycloak.admin.client.resource.UsersResource;
import org.keycloak.common.VerificationException;
import org.keycloak.representations.AccessToken;
import org.keycloak.representations.idm.ClientRepresentation;
import org.keycloak.representations.idm.CredentialRepresentation;
import org.keycloak.representations.idm.RoleRepresentation;
import org.keycloak.representations.idm.UserRepresentation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import it.enea.xlab.GECORegistration.model.user.User;

@Component
public class KeycloakGECOManager {

	@Value("${keycloak.auth-server-url}")
	private String serverUrl;

	@Value("${keycloak.realm}")
	private String realm;

	@Value("${keycloak.resource}")
	private String clientId;

	@Value("${keycloak.credentials.secret}")
	private String clientSecret;

	@Value("${k.adminusername}")
	private String adminUsername;

	@Value("${k.adminpassword}")
	private String adminPassword;

	UserResource userResource;

	@Autowired
	KeycloakClient kc;

	Keycloak keycloak;

	public KeycloakGECOManager() {

	}

	public Boolean isValidToken(String token) {
		try {
			AccessToken aToken = TokenVerifier.create(token, AccessToken.class).getToken();

			return aToken.isActive();

		} catch (VerificationException ve) {
			System.out.println(ve.getLocalizedMessage());
		}
		return true;
	}

//	public String getTokenFromUsernameAndPassword(String username, String password) {
//		String tokenString = "";
//		
////		Keycloak instance = 

////				Keycloak.getInstance(
////						kc."http://keycloak:8080/auth", "myrealm", "user", "user_password","client_id", "client_secret");                                                                                                      
////
////		
////		keycloak.getInstance(
////				tokenString, tokenString, 
////				username, 
////				password, username, password, null, null, true, tokenString)
//		
//		return tokenString;
//	}

	
	public String saveUser(final User user) {
		keycloak = kc.getKeycloakClientLocal();
		
		UserRepresentation userRep = new UserRepresentation();
		userRep.setEnabled(true);
		userRep.setUsername(user.getEmail());
		//userRep.setFirstName(user.getUsername());
		//userRep.setLastName(user.getLastName());
		//userRep.setEmail(user.getLastName());
		
		RealmResource realmResource = keycloak.realm(realm);
		UsersResource usersRessource = realmResource.users();
		Response response = usersRessource.create(userRep);
		String userId = CreatedResponseUtil.getCreatedId(response);
		
		System.out.println("Created user with id = " + userId);
		
		return userId;
		
	}
	
	public void run() {
		keycloak = kc.getKeycloakClientLocal();

		try {
			// Define user
			UserRepresentation user = new UserRepresentation();
			user.setEnabled(true);
			user.setUsername("tester1");
			user.setFirstName("First");
			user.setLastName("Last");
			user.setEmail("gianluca.dagosta@gmail.com");
			user.setAttributes(Collections.singletonMap("origin", Arrays.asList("demo")));

			// Get realm
			RealmResource realmResource = keycloak.realm(realm);
			UsersResource usersRessource = realmResource.users();

			// Create user (requires manage-users role)
			Response response = usersRessource.create(user);
			System.out.printf("Repsonse: %s %s%n", response.getStatus(), response.getStatusInfo());
			System.out.println(response.getLocation());
			String userId = CreatedResponseUtil.getCreatedId(response);

			System.out.printf("User created with userId: %s%n", userId);

			// Define password credential
			CredentialRepresentation passwordCred = new CredentialRepresentation();
			passwordCred.setTemporary(false);
			passwordCred.setType(CredentialRepresentation.PASSWORD);
			passwordCred.setValue("test");

			userResource = usersRessource.get(userId);

			// Set password credential
			userResource.resetPassword(passwordCred);

//     // Get realm role "tester" (requires view-realm role)
			RoleRepresentation testerRealmRole = realmResource.roles()//
					.get("geco_simple_user").toRepresentation();
//
//     // Assign realm role tester to user
			userResource.roles().realmLevel() //
					.add(Arrays.asList(testerRealmRole));
//
//     // Get client
			ClientRepresentation app1Client = realmResource.clients() //
					.findByClientId("GECO").get(0);
//
//     // Get client level role (requires view-clients role)
			// RoleRepresentation userClientRole =
			// realmResource.clients().get(app1Client.getId()) //
			// .roles().get("user").toRepresentation();
//
//     // Assign client level role to user
			// userResource.roles() //
			// .clientLevel(app1Client.getId()).add(Arrays.asList(userClientRole));

			// Send password reset E-Mail
			// VERIFY_EMAIL, UPDATE_PROFILE, CONFIGURE_TOTP, UPDATE_PASSWORD,
			// TERMS_AND_CONDITIONS
			usersRessource.get(userId).executeActionsEmail(Arrays.asList("UPDATE_PASSWORD"));

		} catch (Exception e) {
			System.out.println(e.getLocalizedMessage());
		} finally {
			// Delete User
			userResource.remove();

			System.out.println("User deleted");
		}
		
		
	}
	public void deleteUser(User user) {
		
		RealmResource realmResource = keycloak.realm(realm);
		UsersResource usersRessource = realmResource.users();

		userResource = usersRessource.get(user.getSubject());
		
		userResource.remove();
		
	}
}
