package it.enea.xlab.GECORegistration;

import org.junit.jupiter.api.MethodOrderer;
import static org.hamcrest.Matchers.greaterThan;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.junit.jupiter.api.Order;
import org.junit.runner.RunWith;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.context.annotation.ComponentScan;
import org.springframework.http.MediaType;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.MvcResult;
import org.springframework.test.web.servlet.ResultHandler;

import com.fasterxml.jackson.databind.ObjectMapper;

import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;
import static org.springframework.test.web.servlet.result.MockMvcResultHandlers.*;

import it.enea.xlab.GECORegistration.config.SpringFoxConfig;
import it.enea.xlab.GECORegistration.config.dataDBConfiguration;
import it.enea.xlab.GECORegistration.config.userDBConfiguration;
import it.enea.xlab.GECORegistration.controller.UserController;
import it.enea.xlab.GECORegistration.dto.LocalUserDTO;
import it.enea.xlab.GECORegistration.model.user.User;
import it.enea.xlab.GECORegistration.service.identityManager.IEncryptionManager;
import it.enea.xlab.GECORegistration.service.identityManager.IIdpClientManager;
import it.enea.xlab.GECORegistration.service.identityManager.IdpClientManager;
import it.enea.scp.idp.sdk.model.UserDto;


@RunWith(SpringRunner.class)
@SpringBootTest(classes= {userDBConfiguration.class, SpringFoxConfig.class, dataDBConfiguration.class})
@TestPropertySource(
		locations = "classpath:application-test.properties")
@AutoConfigureMockMvc
@ComponentScan(basePackages = "it.enea.xlab.GECORegistration")
@TestMethodOrder(MethodOrderer.OrderAnnotation.class)
class GecoRegistrationSpringApplicationTests {

	@Value("${testingPhase}")
	private Boolean testingPhase;

	//@Value("${t.userToken}")
	//private String userToken;

	//@Value("${t.adminToken}")
	//private String adminToken;

	@Value("${idp.testAdminEmail}")
	String userAdminEmail;

	@Value("${idp.testAdminPwd}")
	String userAdminPwd;

	@Value("${idp.testUserEmail}")
	String testUserEmail;

	@Value("${idp.testUserPwd}")
	String testUserPwd;

	@Autowired
	private MockMvc mvc;

	@Autowired 
	IIdpClientManager idpClientManager;

	@Autowired
	IEncryptionManager encryptionManager;

	@Autowired
	UserController userController;

	private String token;

	//	@Test
	//	void TestIdp() throws Exception{
	//		String token = idpClientManager.login("gianluca.dagosta@enea.it", "gianluca.dagosta");
	//		
	//		System.out.println(token);
	//		
	//		assert (idpClientManager.isValid(token));
	//		
	//		UserDto user = new UserDto();
	//		
	//		user.setAddress("Via qualsiasi, un po' piu' in la' 32, 00100 Ovunque");
	//		user.setName("cancellami");
	//		user.setSurname("D'Agosta");
	//		user.setPhone("0516098466");
	//		user.setEmail("test@test.com");
	//		user.setEmailConfirmed(true);
	//		user.setPassword("gianluca.dagosta");
	//		user.setOrganization("Comunita' GECO");
	//		user.setPasswordConfirm("gianluca.dagosta");
	//		user.setRole("Citizen");
	//				
	//		assert(idpClientManager.createUser(user));
	//		
	//		UserDto tempUser = idpClientManager.getUser("test@test.com");
	//		tempUser.setAddress("ho traslocato!");
	//		
	//		
	//		
	//		List<UserDto> userList1 = idpClientManager.userList();
	//		
	//		assert(idpClientManager.deleteUser("test@test.com"));
	//		
	//		List<UserDto> userList2 = idpClientManager.userList();
	//		
	//		assert(userList1.size()>userList2.size());
	//		
	//	}

	//	@Test
	//	@Order(1)
	//	void registernewUser() throws Exception{
	//		//Questo test cerca di capire il procedimento per la registrazione di un utente su ID-Provider e nel 
	//		//database della comunità energetica in modo che esista un legame tra i due DB
	//		
	//		UserDto userDto = createFakeUserDto();
	//		User u = new User();
	//		String subject = encryptionManager.encrypt(userDto.getEmail());		
	//		
	//		u.setSubject(subject);
	//		
	//		userController.persistNewUserWithCommunityIdAndRoleId(u, new Long(4), new Long(2));
	//		
	//		System.out.println(u);
	//		
	//	}
	//	
	//	
	//	
	//	
	//	@Test
	//	@Order (2)
	//	void ListAllUsersFromIdp() throws Exception {
	//		
	//		System.out.println(testingPhase);
	//		
	//		System.out.println(adminToken);
	//		
	//		System.out.println(userToken);
	//		
	//		mvc.perform(
	//				get("/idp/usersFromIdp"))
	//				.andExpect(status().isOk())
	//				.andDo(print());		
	//	}
	//	
	//	@Test
	//	@Order(3)
	//	void ListAllUsers() throws Exception {
	//		
	//		System.out.println(testingPhase);
	//		
	//		System.out.println(adminToken);
	//		
	//		System.out.println(userToken);
	//		
	//		mvc.perform(
	//				get("/ListAllUsers"))
	//				.andExpect(status().isOk())
	//				.andDo(print());		
	//	}

	//	@Test
	//	void userFromToken() throws Exception {
	//		
	//		mvc.perform(
	//				post("/userFromToken")
	//				.contentType(MediaType.APPLICATION_JSON)
	//				.param("token", userToken))
	//				.andExpect(status().isCreated());		
	//	}

	//	@Test
	//	void adminFromToken() throws Exception {
	//		
	//		Token token = new Token(adminToken);
	//		
	//		mvc.perform(
	//				post("/userFromToken")
	//				.contentType(MediaType.APPLICATION_JSON)
	//				.param("token", adminToken))
	//				.andExpect(status().isCreated());		
	//	}

	//	@Test
	//	@Order(4)
	//	void getUserFromSubject() throws Exception {
	//		
	//		//String subject = JWTManager.getSubject(userToken);
	//		UserDto userDto = createFakeUserDto();
	//		User u = new User();
	//		String subject = encryptionManager.encrypt(userDto.getEmail());		
	//		
	//		mvc.perform(
	//				get("/userFromSubject")
	//				.contentType(MediaType.APPLICATION_JSON)
	//				.param("subject", subject))
	//				.andExpect(status().isFound())
	//				.andDo(print());
	//	}

	//	@Test
	//	void getUserCommunities() throws Exception {
	//		
	//		Token token = new Token(adminToken);
	//		
	//		mvc.perform(
	//				post("/getusercommunities")
	//				.contentType(MediaType.APPLICATION_JSON)
	//				.param("token", adminToken))
	//				.andExpect(status().isOk());		
	//	}

	//	@Test
	//	void getUsers() throws Exception {
	//		
	//		//Token token = new Token(adminToken);
	//		
	//		mvc.perform(
	//				post("/getusers")
	//				.contentType(MediaType.APPLICATION_JSON)
	//				.param("token", adminToken))
	//				.andExpect(status().isOk())
	//				.andDo(print());		
	//	}





	@Test
	@Order(5)
	void getUserFromEmailAndPassword() throws Exception {



	mvc.perform(
				post("/user/login")
				.contentType(MediaType.APPLICATION_JSON)
				.param("username", userAdminEmail)
				.param("password", userAdminPwd))
		.andExpect(status().isOk())
		.andDo(print());
		
		mvc.perform(
				post("/user/login")
				.contentType(MediaType.APPLICATION_JSON)
				.param("username", testUserEmail)
				.param("password", testUserPwd))
		.andExpect(status().isOk())
		.andDo(print());

		mvc.perform(
				post("/user/login")
				.contentType(MediaType.APPLICATION_JSON)
				.param("username", testUserEmail + "w")
				.param("password", testUserPwd))
		.andExpect(status().isForbidden())
		.andDo(print());
		
//		String userContent ="{\n"
//				+ "  \"name\": \"Gianluca\",\n"
//				+ "  \"surname\": \"D'Agosta\",\n"
//				+ "  \"organization\": \"EnergyCommunity\",\n"
//				+ "  \"email\": \"gianluca.dagosta@cancellamiAppenaPuoi.it\",\n"
//				+ "  \"password\": \"cancellami!\",\n"
//				+ "  \"userroles\": [\n"
//				+ "    {\n"
//				+ "      \"role\": {\n"
//				+ "        \"roleID\": 2,\n"
//				+ "        \"communityID\": 4,\n"
//				+ "        \"roleName\": \"registered\",\n"
//				+ "        \"description\": \"registered\"\n"
//				+ "      }\n"
//				+ "    }\n"
//				+ "  ],\n"
//				+ "  \"usercommunities\": [\n"
//				+ "    {\n"
//				+ "      \"community\": {\n"
//				+ "        \"communityid\": 4,\n"
//				+ "        \"communityName\": \"Farlock\",\n"
//				+ "        \"description\": \"Comunita' di test\",\n"
//				+ "        \"wallet\": \"dhòljhà jpeoqàjàpjfaàkàhlkanhàlkjàoiteàih\"\n"
//				+ "      }\n"
//				+ "    }\n"
//				+ "  ]\n"
//				+ "}";
//		
//		mvc.perform(
//				post("/user/register")
//				.contentType(MediaType.APPLICATION_JSON)
//				.content(userContent))
//		.andExpect(status().isCreated())
//		.andDo(print());
//		
//		String deleteContent = "gianluca.dagosta@cancellamiAppenaPuoi.it";
//		
//		mvc.perform(
//				delete("/user/delete")
//				.contentType(MediaType.APPLICATION_JSON)
//				.content(deleteContent))
//		.andExpect(status().isOk())
//		.andDo(print());
		
	}

	@Test
	@Order(6)
	void getData() throws Exception {

		Handler handler = new Handler();

		mvc.perform(
				post("/user/login")
				.contentType(MediaType.APPLICATION_JSON)
				.param("username", testUserEmail)
				.param("password", testUserPwd))
		.andExpect(status().isOk())
		.andDo(handler)
		.andDo(print());

		
		mvc.perform(
				post("/user/getUserIdFromToken")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token))
		.andExpect(status().isOk())
		.andDo(print());
		
		mvc.perform(
				post("/user/isTokenValid")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token))
		.andExpect(status().isOk())
		.andDo(print());
		
		mvc.perform(
				post("/data/getListOfFunctions")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token))
		.andExpect(status().isOk())
		.andDo(print());

		String content ="{"
				+ "\"communityid\": 2,\n"
				+ " \"queryid\":2,\n"
				+ "\" endDate\": \"2021-09-22 23:59:59\",\n"
				+ "\" startDate\":\"2021-09-22 00:00:00\"\n"
				+ "}";



		mvc.perform(
				post("/data/getData")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token)
				.header("Content-Language", "it_IT")
				.content (content))				
		.andExpect(status().isOk())
		//.andExpect(jsonPath("$.result", greaterThan(2)))
		.andDo(print());

		content ="{\"communityid\": 2,\n"
				+ " \"queryid\":1,\n"
				+ "\" maxValue\": \"3000\",\n"
				+ "\" minValue\":\"0\"\n"
				+ "}";


		mvc.perform(
				post("/data/getData")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token)
				.header("Content-Language", "it_IT")
				.content(content))
		.andExpect(status().isOk())
		.andDo(print());

		content ="{\"communityid\": 2,\n"
				+ " \"queryid\":5,\n"
				+ "\" endDate\": \"2021-09-22 23:59:59\",\n"
				+ "\" startDate\":\"2021-09-21 00:00:00\"\n"
				+ "}";

		mvc.perform(
				post("/data/getData")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token)
				.header("Content-Language", "it_IT")
				.content(content))
		.andExpect(status().isOk())
		.andDo(print());

		content ="{\"communityid\": 2,\n"
				+ " \"queryid\":2,\n"
				+ "\" endDate\": \"2021-09-22 23:59:59\",\n"
				+ "\" startDate\":\"2021-09-21 00:00:00\"\n"
				+ "}";


		mvc.perform(
				post("/data/getData")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token)
				.header("Content-Language", "it_IT")
				.content(content))
		.andExpect(status().isOk())
		.andDo(print());

		content ="{\"communityid\": 2,\n"
				+ " \"queryid\":6\n"
				+ "}";

		mvc.perform(
				post("/data/getData")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token)
				.header("Content-Language", "it_IT")
				.content(content))
		.andExpect(status().isOk())
		.andDo(print());

		content ="{\"queryid\":4\n"
				+ "}";

		mvc.perform(
				post("/data/getData")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token)
				.content(content))
		.andExpect(status().isOk())
		.andDo(print());

		mvc.perform(
				post("/user/logout")
				.contentType(MediaType.APPLICATION_JSON)
				.header("Authorization", token)
				.param("token", token))
		.andExpect(status().isOk())
		.andDo(print());

	}

	/*
	 * Questo test verifica le funzioni di estrazione dei dati dal DB.
	 * Devono ancora implementare la gestione degli utenti.
	 * 
	 */
	//	@Test
	//	@Order(12)
	//	void getConsumptionData() throws Exception {
	//		mvc.perform(
	//				post("/data/getInstantConsumption"))
	//				.andExpect(status().isOk())
	//				.andDo(print())
	//				.andExpect(jsonPath("$..property").exists())
	//			    .andExpect(jsonPath("$..val").isNotEmpty());
	//		
	//		mvc.perform(
	//				post("/data/todayConsumption"))
	//				.andExpect(status().isOk())
	//				.andDo(print())
	//				.andExpect(jsonPath("$..property").exists())
	//			    .andExpect(jsonPath("$..val").isNotEmpty());	
	//		
	//		mvc.perform(
	//				post("/data/getConsumptionInAPeriod")
	//				.queryParam("startDate", "2021-06-01 00:00:00")
	//				.queryParam("endDate", "2021-06-07  00:00:00")
	//				)
	//				.andExpect(status().isOk())
	//				.andDo(print())
	//				.andExpect(jsonPath("$..property").exists())
	//			    .andExpect(jsonPath("$..val").isNotEmpty());	
	//		
	//		
	//	}


	private class Token  {
		String token;

		public Token(String token) {
			this.token = token;
		}

		public String getToken() {
			return this.token;
		}

		public void setToken (String token) {
			this.token = token;
		}
	}

	private UserDto createFakeUserDto(){

		UserDto user = new UserDto();

		user.setAddress("Via qualsiasi, un po' piu' in la' 32, 00100 Ovunque");
		user.setName("cancellami");
		user.setSurname("D'Agosta");
		user.setPhone("0516098466");
		user.setEmail("utente.ec@test.it");
		user.setEmailConfirmed(true);
		user.setPassword("gianluca.dagosta");
		user.setOrganization("Comunita' GECO");
		user.setPasswordConfirm("gianluca.dagosta");
		user.setRole("Citizen");

		return user;

	}

	private class Handler implements ResultHandler{

		@Override
		public void handle(MvcResult result) throws Exception {
			String pippo = result.getResponse().getContentAsString();
			LocalUserDTO user = new ObjectMapper().readValue(pippo, LocalUserDTO.class);
			token = user.getToken().getValue();
			//System.out.println(result);

		}
	}
}
